@@ -220,3 +220,17 @@ debian_installer = deployment_data_dict({  # pylint: disable=invalid-name
     'lava_test_results_dir': '/lava-%s',
     'lava_test_shell_file': '~/.bashrc',
 })
+
+agl = deployment_data_dict({
+    'TESTER_PS1': "linaro-test [rc=$(echo \$?)]# ",
+    'TESTER_PS1_PATTERN': "linaro-test \[rc=(\d+)\]# ",
+    'TESTER_PS1_INCLUDES_RC': True,
+    'boot_cmds': 'boot_cmds_oe',
+
+    # for lava-test-shell
+    'distro': 'oe',
+    'lava_test_sh_cmd': '/bin/bash',
+    'lava_test_dir': '/home/root/lava-%s',
+    'lava_test_results_part_attr': 'root_part',
+    'lava_test_results_dir': '/home/root/lava-%s',
+})
