@@ -32,6 +32,7 @@ from lava_dispatcher.pipeline.actions.boot import (
     BootAction,
     AutoLoginAction,
     BootloaderCommandOverlay,
+    BootloaderSecondaryMedia,
     BootloaderCommandsAction,
     OverlayUnpack,
 )
@@ -39,6 +40,7 @@ from lava_dispatcher.pipeline.actions.boot.uefi_menu import (
     UEFIMenuInterrupt,
     UefiMenuSelector
 )
+from lava_dispatcher.pipeline.actions.boot.fastboot import WaitFastBootInterrupt
 from lava_dispatcher.pipeline.actions.boot.environment import ExportDeviceEnvironment
 from lava_dispatcher.pipeline.shell import ExpectShellSession
 from lava_dispatcher.pipeline.connections.serial import ConnectDevice
@@ -48,18 +50,37 @@ from lava_dispatcher.pipeline.power import (
 )
 
 
-def bootloader_accepts(device, parameters):
-    if 'method' not in parameters:
-        raise ConfigurationError("method not specified in boot parameters")
-    if parameters["method"] not in ["grub", "grub-efi"]:
-        return False
-    if 'actions' not in device:
-        raise ConfigurationError("Invalid device configuration")
-    if 'boot' not in device['actions']:
-        return False
-    if 'methods' not in device['actions']['boot']:
-        raise ConfigurationError("Device misconfiguration")
-    return True
+class GrubSequence(Boot):
+
+    compatibility = 3
+
+    def __init__(self, parent, parameters):
+        super(GrubSequence, self).__init__(parent)
+        self.action = GrubSequenceAction()
+        self.action.section = self.action_type
+        self.action.job = self.job
+        parent.add_action(self.action, parameters)
+
+    @classmethod
+    def accepts(cls, device, parameters):
+        if 'method' not in parameters:
+            raise ConfigurationError("method not specified in boot parameters")
+        if parameters["method"] not in ["grub", "grub-efi"]:
+            return False, '"method" was not "grub" or "grub-efi"'
+        if 'actions' not in device:
+            raise ConfigurationError("Invalid device configuration")
+        if 'boot' not in device['actions']:
+            return False, '"boot" was not in the device configuration actions'
+        if 'methods' not in device['actions']['boot']:
+            raise ConfigurationError("Device misconfiguration")
+        params = device['actions']['boot']['methods']
+        if 'grub' not in params:
+            return False, '"grub" was not in the device configuration boot methods'
+        if 'grub-efi' in params:
+            return False, '"grub-efi" was not in the device configuration boot methods'
+        if 'sequence' in params['grub']:
+            return True, 'accepted'
+        return False, '"sequence" not in device configuration boot methods'
 
 
 class Grub(Boot):
@@ -75,10 +96,78 @@ class Grub(Boot):
 
     @classmethod
     def accepts(cls, device, parameters):
-        if not bootloader_accepts(device, parameters):
-            return False
+        if 'method' not in parameters:
+            raise ConfigurationError("method not specified in boot parameters")
+        if parameters["method"] not in ["grub", "grub-efi"]:
+            return False, '"method" was not "grub" or "grub-efi"'
+        if 'actions' not in device:
+            raise ConfigurationError("Invalid device configuration")
+        if 'boot' not in device['actions']:
+            return False, '"boot" was not in the device configuration actions'
+        if 'methods' not in device['actions']['boot']:
+            raise ConfigurationError("Device misconfiguration")
         params = device['actions']['boot']['methods']
-        return 'grub' in params or 'grub-efi' in params
+        if 'grub' in params and 'sequence' in params['grub']:
+            return False, '"sequence" was in "grub" parameters'
+        if 'grub' in params or 'grub-efi' in params:
+            return True, 'accepted'
+        else:
+            return False, '"grub" or "grub-efi" was not in the device configuration boot methods'
+
+
+def _grub_sequence_map(sequence):
+    """Maps grub sequence with corresponding class."""
+    sequence_map = {
+        'wait-fastboot-interrupt': (WaitFastBootInterrupt, 'grub'),
+        'auto-login': (AutoLoginAction, None),
+        'shell-session': (ExpectShellSession, None),
+        'export-env': (ExportDeviceEnvironment, None),
+    }
+    return sequence_map.get(sequence, (None, None))
+
+
+class GrubSequenceAction(BootAction):
+
+    def __init__(self):
+        super(GrubSequenceAction, self).__init__()
+        self.name = "grub-sequence-action"
+        self.description = "grub boot sequence"
+        self.summary = "run grub boot using specified sequence of actions"
+        self.expect_shell = False
+
+    def validate(self):
+        super(GrubSequenceAction, self).validate()
+        sequences = self.job.device['actions']['boot']['methods']['grub'].get(
+            'sequence', [])
+        for sequence in sequences:
+            if not _grub_sequence_map(sequence):
+                self.errors = "Unknown boot sequence '%s'" % sequence
+
+    def populate(self, parameters):
+        super(GrubSequenceAction, self).populate(parameters)
+        self.internal_pipeline = Pipeline(parent=self, job=self.job,
+                                          parameters=parameters)
+        sequences = self.job.device['actions']['boot']['methods']['grub'].get(
+            'sequence', [])
+        for sequence in sequences:
+            mapped = _grub_sequence_map(sequence)
+            if mapped[1]:
+                self.internal_pipeline.add_action(
+                    mapped[0](type=mapped[1]))
+            elif mapped[0]:
+                self.internal_pipeline.add_action(mapped[0]())
+        if self.has_prompts(parameters):
+            self.internal_pipeline.add_action(AutoLoginAction())
+            if self.test_has_shell(parameters):
+                self.internal_pipeline.add_action(ExpectShellSession())
+                if 'transfer_overlay' in parameters:
+                    self.internal_pipeline.add_action(OverlayUnpack())
+                self.internal_pipeline.add_action(ExportDeviceEnvironment())
+        else:
+            if self.has_boot_finished(parameters):
+                self.logger.debug("Doing a boot without a shell (installer)")
+                self.internal_pipeline.add_action(InstallerWait())
+                self.internal_pipeline.add_action(PowerOff())
 
 
 class GrubMainAction(BootAction):
@@ -92,10 +181,11 @@ class GrubMainAction(BootAction):
     def populate(self, parameters):
         self.expect_shell = parameters.get('expect_shell', True)
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        self.internal_pipeline.add_action(BootloaderSecondaryMedia())
         self.internal_pipeline.add_action(BootloaderCommandOverlay())
         self.internal_pipeline.add_action(ConnectDevice())
         # FIXME: reset_device is a hikey hack due to fastboot/OTG issues
-        # remove as part of LAVA-940
+        # remove as part of LAVA-940 - convert to use fastboot-sequence
         reset_device = self.job.device['actions']['boot']['methods'].get('grub-efi', {}).get('reset_device', True)
         if parameters['method'] == 'grub-efi' and reset_device:
             # added unless the device specifies not to reset the device in grub.
@@ -139,14 +229,13 @@ class BootloaderInterrupt(Action):
 
     def validate(self):
         super(BootloaderInterrupt, self).validate()
-        hostname = self.job.device['hostname']
         if self.job.device.connect_command is '':
-            self.errors = "Unable to connect to device %s" % hostname
+            self.errors = "Unable to connect to device"
         device_methods = self.job.device['actions']['boot']['methods']
         if self.parameters['method'] == 'grub-efi' and 'grub-efi' in device_methods:
             self.type = 'grub-efi'
         if 'bootloader_prompt' not in device_methods[self.type]['parameters']:
-            self.errors = "Missing bootloader prompt for device"
+            self.errors = "[%s] Missing bootloader prompt for device" % self.name
 
     def run(self, connection, max_end_time, args=None):
         if not connection:
