@@ -53,8 +53,8 @@ class BootLxc(Boot):
     def accepts(cls, device, parameters):
         if 'method' in parameters:
             if parameters['method'] == 'lxc':
-                return True
-        return False
+                return True, 'accepted'
+        return False, '"method" was not in parameters or "method" was not "lxc"'
 
 
 class BootLxcAction(BootAction):
