@@ -33,6 +33,7 @@ from lava_dispatcher.pipeline.actions.boot import (
     AutoLoginAction,
     BootloaderCommandOverlay,
     BootloaderCommandsAction,
+    BootloaderSecondaryMedia,
     OverlayUnpack,
 )
 from lava_dispatcher.pipeline.actions.boot.environment import ExportDeviceEnvironment
@@ -40,22 +41,7 @@ from lava_dispatcher.pipeline.shell import ExpectShellSession
 from lava_dispatcher.pipeline.connections.lxc import ConnectLxc
 from lava_dispatcher.pipeline.connections.serial import ConnectDevice
 from lava_dispatcher.pipeline.power import ResetDevice
-
-
-def uboot_accepts(device, parameters):
-    if 'method' not in parameters:
-        raise ConfigurationError("method not specified in boot parameters")
-    if parameters['method'] != 'u-boot':
-        return False
-    if 'commands' not in parameters:
-        raise ConfigurationError("commands not specified in boot parameters")
-    if 'actions' not in device:
-        raise ConfigurationError("Invalid device configuration")
-    if 'boot' not in device['actions']:
-        return False
-    if 'methods' not in device['actions']['boot']:
-        raise ConfigurationError("Device misconfiguration")
-    return True
+from lava_dispatcher.pipeline.utils.strings import map_kernel_uboot
 
 
 class UBoot(Boot):
@@ -79,9 +65,14 @@ class UBoot(Boot):
 
     @classmethod
     def accepts(cls, device, parameters):
-        if not uboot_accepts(device, parameters):
-            return False
-        return 'u-boot' in device['actions']['boot']['methods']
+        if parameters['method'] != 'u-boot':
+            return False, '"method" was not "u-boot"'
+        if 'commands' not in parameters:
+            raise ConfigurationError("commands not specified in boot parameters")
+        if 'u-boot' in device['actions']['boot']['methods']:
+            return True, 'accepted'
+        else:
+            return False, '"u-boot" was not in the device configuration boot methods'
 
 
 class UBootAction(BootAction):
@@ -143,10 +134,7 @@ class UBootRetry(BootAction):
 
     def run(self, connection, max_end_time, args=None):
         connection = super(UBootRetry, self).run(connection, max_end_time, args)
-        # Log an error only when needed
         self.set_namespace_data(action='shared', label='shared', key='connection', value=connection)
-        if self.errors:
-            self.logger.error(self.errors)
         return connection
 
 
@@ -162,9 +150,8 @@ class UBootInterrupt(Action):
 
     def validate(self):
         super(UBootInterrupt, self).validate()
-        hostname = self.job.device['hostname']
         if self.job.device.connect_command is '':
-            self.errors = "Unable to connect to device %s" % hostname
+            self.errors = "Unable to connect to device %s"
         device_methods = self.job.device['actions']['boot']['methods']
         if 'bootloader_prompt' not in device_methods['u-boot']['parameters']:
             self.errors = "Missing bootloader prompt for device"
@@ -191,7 +178,7 @@ class UBootInterrupt(Action):
         return connection
 
 
-class UBootSecondaryMedia(Action):
+class UBootSecondaryMedia(BootloaderSecondaryMedia):
     """
     Idempotent action which sets the static data only used when this is a boot of secondary media
     already deployed.
@@ -203,30 +190,23 @@ class UBootSecondaryMedia(Action):
         self.description = "let uboot know where to find the kernel in the image on secondary media"
 
     def validate(self):
-        super(UBootSecondaryMedia, self).validate()
-        if 'media' not in self.job.device['parameters']:
+        if 'media' not in self.job.device.get('parameters', []):
             return
         media_keys = self.job.device['parameters']['media'].keys()
         if self.parameters['commands'] not in list(media_keys):
             return
-        if 'kernel' not in self.parameters:
-            self.errors = "Missing kernel location"
+        super(UBootSecondaryMedia, self).validate()
         if 'kernel_type' not in self.parameters:
             self.errors = "Missing kernel_type for secondary media boot"
-        # ramdisk does not have to be specified, nor dtb
-        if 'root_uuid' not in self.parameters:
-            # FIXME: root_node also needs to be supported
-            self.errors = "Missing UUID of the roofs inside the deployed image"
-        if 'boot_part' not in self.parameters:
-            self.errors = "Missing boot_part for the partition number of the boot files inside the deployed image"
+        self.logger.debug("Mapping kernel_type: %s", self.parameters['kernel_type'])
+        bootcommand = map_kernel_uboot(self.parameters['kernel_type'], self.job.device.get('parameters', None))
+        self.logger.debug("Using bootcommand: %s", bootcommand)
         self.set_namespace_data(
             action='uboot-prepare-kernel', label='kernel-type',
             key='kernel-type', value=self.parameters.get('kernel_type', ''))
+        self.set_namespace_data(
+            action='uboot-prepare-kernel', label='bootcommand', key='bootcommand', value=bootcommand)
 
-        self.set_namespace_data(action=self.name, label='file', key='kernel', value=self.parameters.get('kernel', ''))
-        self.set_namespace_data(action=self.name, label='file', key='ramdisk', value=self.parameters.get('ramdisk', ''))
-        self.set_namespace_data(action=self.name, label='file', key='dtb', value=self.parameters.get('dtb', ''))
-        self.set_namespace_data(action=self.name, label='uuid', key='root', value=self.parameters['root_uuid'])
         media_params = self.job.device['parameters']['media'][self.parameters['commands']]
         if self.get_namespace_data(action='storage-deploy', label='u-boot', key='device') not in media_params:
             self.errors = "%s does not match requested media type %s" % (
@@ -289,6 +269,4 @@ class UBootEnterFastbootAction(BootAction):
                 self.wait(connection)
                 i += 1
 
-        if self.errors:
-            self.logger.error(self.errors)
         return connection
