@@ -23,12 +23,14 @@
 
 # pylint: disable=unused-import
 
+from lava_dispatcher.pipeline.actions.deploy.docker import Docker
 from lava_dispatcher.pipeline.actions.deploy.image import DeployImages
-from lava_dispatcher.pipeline.actions.deploy.tftp import Tftp
-from lava_dispatcher.pipeline.actions.deploy.removable import MassStorage
-from lava_dispatcher.pipeline.actions.deploy.ssh import Ssh
+from lava_dispatcher.pipeline.actions.deploy.iso import DeployIso
 from lava_dispatcher.pipeline.actions.deploy.fastboot import Fastboot
 from lava_dispatcher.pipeline.actions.deploy.lxc import Lxc
-from lava_dispatcher.pipeline.actions.deploy.iso import DeployIso
+from lava_dispatcher.pipeline.actions.deploy.removable import MassStorage
+from lava_dispatcher.pipeline.actions.deploy.nbd import Nbd
 from lava_dispatcher.pipeline.actions.deploy.nfs import Nfs
+from lava_dispatcher.pipeline.actions.deploy.ssh import Ssh
+from lava_dispatcher.pipeline.actions.deploy.tftp import Tftp
 from lava_dispatcher.pipeline.actions.deploy.vemsd import VExpressMsd
