@@ -52,10 +52,6 @@ from lava_dispatcher.pipeline.utils.constants import (
     DEFAULT_V1_PATTERN,
     DEFAULT_V1_FIXUP,
 )
-from lava_dispatcher.pipeline.utils.udev import (
-    get_udev_devices,
-    usb_device_wait,
-)
 if sys.version > '3':
     from functools import reduce  # pylint: disable=redefined-builtin
 
@@ -75,7 +71,9 @@ class TestShell(LavaTest):
 
     @classmethod
     def accepts(cls, device, parameters):  # pylint: disable=unused-argument
-        return ('definition' in parameters) or ('definitions' in parameters)
+        if ('definition' in parameters) or ('definitions' in parameters):
+            return True, 'accepted'
+        return False, '"definition" or "definitions" not in parameters'
 
     @classmethod
     def needs_deployment_data(cls):
@@ -195,7 +193,7 @@ class TestShellAction(TestAction):
         self._reset_patterns()
         super(TestShellAction, self).validate()
 
-    def run(self, connection, max_end_time, args=None):
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals
         """
         Common run function for subclasses which define custom patterns
         """
@@ -237,6 +235,10 @@ class TestShellAction(TestAction):
             for command in pre_command_list:
                 connection.sendline(command, delay=self.character_delay)
 
+        if lava_test_results_dir is None:
+            raise JobError("Nothing to run. Maybe the 'deploy' stage is missing, "
+                           "otherwise this is a bug which should be reported.")
+
         self.logger.debug("Using %s" % lava_test_results_dir)
         connection.sendline('ls -l %s/' % lava_test_results_dir, delay=self.character_delay)
         if lava_test_sh_cmd:
@@ -244,7 +246,7 @@ class TestShellAction(TestAction):
 
         try:
             feedbacks = []
-            for feedback_ns in self.data.keys():
+            for feedback_ns in self.data.keys():  # pylint: disable=no-member
                 if feedback_ns == self.parameters.get('namespace'):
                     continue
                 feedback_connection = self.get_namespace_data(
@@ -459,33 +461,6 @@ class TestShellAction(TestAction):
             name = "testset_" + action.lower()
         return name
 
-    def signal_lxc_add(self):
-        # the lxc namespace may not be accessible here depending on the
-        # lava-test-shell action namespace.
-        lxc_name = None
-        protocols = [protocol for protocol in self.job.protocols if protocol.name == LxcProtocol.name]
-        protocol = protocols[0] if protocols else None
-        if protocol:
-            lxc_name = protocol.lxc_name
-        if not lxc_name:
-            self.logger.debug("No LXC device requested")
-            return False
-        self.logger.info(
-            "Get USB device(s) using: %s",
-            yaml.dump(self.job.device.get('device_info', [])).strip()
-        )
-        device_paths = get_udev_devices(self.job, self.logger)
-        if not device_paths:
-            self.logger.warning("No USB devices added to the LXC.")
-            return False
-        for device in device_paths:
-            lxc_cmd = ['lxc-device', '-n', lxc_name, 'add', device]
-            log = self.run_command(lxc_cmd)
-            self.logger.debug(log)
-            self.logger.debug("%s: device %s added", lxc_name,
-                              device)
-        return True
-
     @nottest
     def pattern_test_case(self, test_connection):
         match = test_connection.match
@@ -566,11 +541,6 @@ class TestShellAction(TestAction):
                 ret = self.signal_test_set(params)
                 if ret:
                     name = ret
-            elif name == "LXCDEVICEADD":
-                self.signal_lxc_add()
-            elif name == "LXCDEVICEWAITADD":
-                self.logger.info("Waiting for USB device(s) ...")
-                usb_device_wait(self.job, device_actions=['add'])
 
             self.signal_director.signal(name, params)
             ret_val = True
@@ -626,6 +596,7 @@ class TestShellAction(TestAction):
                 except TypeError as exc:
                     # handle serial corruption which can overlap kernel messages onto test output.
                     self.logger.exception(str(exc))
+                    raise TestError("Unable to handle the test shell signal correctly: %s" % str(exc))
                 except JobError as exc:
                     self.logger.error("job error: handling signal %s failed: %s", name, exc)
                     return False
