@@ -115,18 +115,17 @@ class JobParser(object):
                 job.timeout = Timeout(data['job_name'], duration)
 
     # pylint: disable=too-many-locals,too-many-statements
-    def parse(self, content, device, job_id, zmq_config, dispatcher_config,
+    def parse(self, content, device, job_id, logger, dispatcher_config,
               output_dir=None, env_dut=None):
         self.loader = yaml.Loader(content)
         self.loader.compose_node = self.compose_node
         self.loader.construct_mapping = self.construct_mapping
         data = self.loader.get_single_data()
-        job = Job(job_id, data, zmq_config)
+        job = Job(job_id, data, logger)
         test_counts = {}
         job.device = device
         job.parameters['output_dir'] = output_dir
         job.parameters['env_dut'] = env_dut
-        job.parameters['target'] = device.target
         # Load the dispatcher config
         job.parameters['dispatcher'] = {}
         if dispatcher_config is not None:
@@ -134,9 +133,6 @@ class JobParser(object):
             if isinstance(config, dict):
                 job.parameters['dispatcher'] = config
 
-        # Setup the logging now that we have the parameters
-        job.setup_logging()
-
         level_tuple = Protocol.select_all(job.parameters)
         # sort the list of protocol objects by the protocol class level.
         job.protocols = [item[0](job.parameters, job_id) for item in sorted(level_tuple, key=lambda level_tuple: level_tuple[1])]
@@ -194,7 +190,7 @@ class JobParser(object):
                     pipeline.add_action(action)
 
                 else:
-                    raise JobError("Unknown action name '%'" % name)
+                    raise JobError("Unknown action name '%s'" % name)
 
         # there's always going to need to be a finalize_process action
         finalize = FinalizeAction()
