@@ -22,9 +22,9 @@
 import re
 import os
 import yaml
-import pexpect
 import logging
 import traceback
+import subprocess
 from lava_dispatcher.pipeline.connection import Protocol
 from lava_dispatcher.pipeline.action import (
     InfrastructureError,
@@ -33,10 +33,10 @@ from lava_dispatcher.pipeline.action import (
     JobError,
     Timeout,
 )
-from lava_dispatcher.pipeline.shell import ShellCommand
 from lava_dispatcher.pipeline.utils.constants import (
     LAVA_LXC_TIMEOUT,
     LXC_PATH,
+    UDEV_RULES_DIR,
 )
 from lava_dispatcher.pipeline.utils.filesystem import lxc_path
 
@@ -95,21 +95,31 @@ class LxcProtocol(Protocol):  # pylint: disable=too-many-instance-attributes
 
     def _api_select(self, data, action=None):
         if not data:
-            raise TestError("Protocol called without any data")
+            raise TestError("[%s] Protocol called without any data." % self.name)
         if not action:
             raise LAVABug('LXC protocol needs to be called from an action.')
         for item in data:
             if 'request' not in item:
-                raise LAVABug("Malformed protocol request data.")
+                raise LAVABug("[%s] Malformed protocol request data." % self.name)
             if 'pre-os-command' in item['request']:
-                action.logger.info("Running pre OS command via protocol.")
+                action.logger.info("[%s] Running pre OS command via protocol.", self.name)
                 command = action.job.device.pre_os_command
                 if not action.run_command(command.split(' '), allow_silent=True):
                     raise InfrastructureError("%s failed" % command)
+                continue
+            elif 'pre-power-command' in item['request']:
+                action.logger.info("[%s] Running pre-power-command via protocol.", self.name)
+                command = action.job.device.pre_power_command
+                if not action.run_command(command.split(' '), allow_silent=True):
+                    raise InfrastructureError("%s failed" % command)
+                continue
+            else:
+                raise JobError("[%s] Unrecognised protocol request: %s" % (self.name, item))
 
     def __call__(self, *args, **kwargs):
         action = kwargs.get('action', None)
         logger = action.logger if action else logging.getLogger("dispatcher")
+        self.logger.debug("[%s] Checking protocol data for %s", action.name, self.name)
         try:
             return self._api_select(args, action=action)
         except yaml.YAMLError as exc:
@@ -117,67 +127,61 @@ class LxcProtocol(Protocol):  # pylint: disable=too-many-instance-attributes
             logger.exception(msg)
             raise JobError("Invalid call to %s %s" % (self.name, exc))
 
+    def _call_handler(self, command):
+        try:
+            self.logger.debug("%s protocol: executing '%s'", self.name, command)
+            output = subprocess.check_output(command.split(' '),
+                                             stderr=subprocess.STDOUT)
+            if output:
+                self.logger.debug(output)
+        except subprocess.CalledProcessError:
+            self.logger.debug("%s protocol: FAILED executing '%s'",
+                              self.name, command)
+
     def finalise_protocol(self, device=None):
         """Called by Finalize action to power down and clean up the assigned
         device.
         """
         # Reboot devices to bootloader if required, based on the availability
-        # of power cycle option and adb_serial_number.
+        # of adb_serial_number.
         # Do not reboot to bootloader if 'reboot_to_fastboot' is set to
         # 'false' in job definition.
         if self.fastboot_reboot:
-            if 'adb_serial_number' in device and hasattr(device, 'power_state'):
-                if device.power_state not in ['on', 'off']:
-                    reboot_cmd = "lxc-attach -n {0} -- adb reboot bootloader".format(self.lxc_name)
-                    self.logger.debug("%s protocol: executing '%s'", self.name,
-                                      reboot_cmd)
-                    shell = ShellCommand("%s\n" % reboot_cmd,
-                                         self.system_timeout,
-                                         logger=self.logger)
-                    # execute the command.
-                    shell.expect(pexpect.EOF)
-                    if shell.exitstatus:
-                        self.logger.debug("%s command exited %d: %s",
-                                          reboot_cmd,
-                                          shell.exitstatus, shell.readlines())
+            if 'adb_serial_number' in device:
+                reboot_cmd = "lxc-attach -n {0} -- adb reboot bootloader".format(self.lxc_name)
+                self._call_handler(reboot_cmd)
         else:
             self.logger.info("%s protocol: device not rebooting to fastboot",
                              self.name)
 
         # Stop the container.
         self.logger.debug("%s protocol: issue stop", self.name)
-        cmd = "lxc-stop -n {0} -k".format(self.lxc_name)
-        shell = ShellCommand("%s\n" % cmd, self.system_timeout,
-                             logger=self.logger)
-        # execute the command.
-        shell.expect(pexpect.EOF)
-        if shell.exitstatus:
-            self.logger.debug(
-                "%s command exited %d: %s" % (cmd, shell.exitstatus,
-                                              shell.readlines()))
+        stop_cmd = "lxc-stop -n {0} -k".format(self.lxc_name)
+        self._call_handler(stop_cmd)
         # Check if the container should persist and skip destroying it.
         if self.persistence:
             self.logger.debug("%s protocol: persistence requested",
                               self.name)
         else:
-            self.logger.debug("%s protocol: destroy", self.name)
+            self.logger.debug("%s protocol: issue destroy", self.name)
             if self.custom_lxc_path:
                 abs_path = os.path.realpath(os.path.join(LXC_PATH,
                                                          self.lxc_name))
-                cmd = "lxc-destroy -n {0} -f -P {1}".format(
+                destroy_cmd = "lxc-destroy -n {0} -f -P {1}".format(
                     self.lxc_name, os.path.dirname(abs_path))
             else:
-                cmd = "lxc-destroy -n {0} -f".format(self.lxc_name)
-                self.logger.debug("%s protocol: executing '%s'",
-                                  self.name, cmd)
-            shell = ShellCommand("%s\n" % cmd, self.system_timeout,
-                                 logger=self.logger)
-            # execute the command.
-            shell.expect(pexpect.EOF)
-            if shell.exitstatus:
-                self.logger.debug(
-                    "%s command exited %d: %s" % (cmd, shell.exitstatus,
-                                                  shell.readlines()))
+                destroy_cmd = "lxc-destroy -n {0} -f".format(self.lxc_name)
+            self._call_handler(destroy_cmd)
             if self.custom_lxc_path and not self.persistence:
                 os.remove(os.path.join(LXC_PATH, self.lxc_name))
+        # Remove udev rule which added device to the container and then reload
+        # udev rules.
+        rules_file = os.path.join(UDEV_RULES_DIR,
+                                  '100-' + self.lxc_name + '.rules')
+        if os.path.exists(rules_file):
+            os.remove(rules_file)
+            self.logger.debug("%s protocol: removed udev rules '%s'",
+                              self.name, rules_file)
+        reload_cmd = "udevadm control --reload-rules"
+        self._call_handler(reload_cmd)
         self.logger.debug("%s protocol finalised.", self.name)
