@@ -91,12 +91,21 @@ class ShellCommand(pexpect.spawn):  # pylint: disable=too-many-public-methods
             raise LAVABug("ShellCommand needs a timeout set by the calling Action")
         if not logger:
             raise LAVABug("ShellCommand needs a logger")
-        pexpect.spawn.__init__(
-            self, command,
-            timeout=lava_timeout.duration,
-            cwd=cwd,
-            logfile=ShellLogger(logger),
-        )
+        if sys.version_info[0] == 2:
+            pexpect.spawn.__init__(
+                self, command,
+                timeout=lava_timeout.duration,
+                cwd=cwd,
+                logfile=ShellLogger(logger),
+            )
+        elif sys.version_info[0] == 3:
+            pexpect.spawn.__init__(
+                self, command,
+                timeout=lava_timeout.duration,
+                cwd=cwd,
+                logfile=ShellLogger(logger),
+                encoding='utf-8',
+            )
         self.name = "ShellCommand"
         self.logger = logger
         # set a default newline character, but allow actions to override as neccessary
@@ -231,8 +240,6 @@ class ShellSession(Connection):
                 else:
                     # TODO: is someone expecting pexpect.TIMEOUT?
                     raise
-            except KeyboardInterrupt:
-                raise
 
     def wait(self, max_end_time=None):
         """
@@ -250,8 +257,6 @@ class ShellSession(Connection):
             return self.raw_connection.expect(self.prompt_str, timeout=timeout)
         except (TestError, pexpect.TIMEOUT):
             raise JobError("wait for prompt timed out")
-        except KeyboardInterrupt:
-            raise
 
     def listen_feedback(self, timeout):
         """
@@ -263,8 +268,6 @@ class ShellSession(Connection):
             self.raw_connection.logfile.is_feedback = True
             return self.raw_connection.expect([pexpect.EOF, pexpect.TIMEOUT],
                                               timeout=timeout)
-        except KeyboardInterrupt:
-            raise
         finally:
             self.raw_connection.logfile.is_feedback = False
 
