@@ -72,8 +72,6 @@ class TestJobDeviceParameters(StdoutTestCase):  # pylint: disable=too-many-publi
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         self.assertIn('power_state', device)
         self.assertFalse(hasattr(device, 'power_state'))
-        self.assertFalse(hasattr(device, 'hostname'))
-        self.assertIn('hostname', device)
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(sample_job_data, device, 4212, None, "")
