@@ -144,6 +144,24 @@ class TestFastbootDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
         for calling in select.parameters['protocols'][LxcProtocol.name]:
             self.assertEqual(calling['action'], select.name)
             self.assertEqual(calling['request'], 'pre-os-command')
+        deploy = [action for action in job.pipeline.actions if action.name == 'fastboot-deploy'][0]
+        self.assertIn(LxcProtocol.name, deploy.parameters.keys())
+        self.assertIn('protocols', deploy.parameters.keys())
+        self.assertIn(LxcProtocol.name, deploy.parameters['protocols'].keys())
+        self.assertEqual(len(deploy.parameters['protocols'][LxcProtocol.name]), 1)
+        for calling in deploy.parameters['protocols'][LxcProtocol.name]:
+            self.assertEqual(calling['action'], deploy.name)
+            self.assertEqual(calling['request'], 'pre-power-command')
+        pair = ['pre-os-command', 'pre-power-command']
+        action_list = {jaction.keys()[0] for jaction in job.parameters['actions']}
+        block = job.parameters['actions']
+        for action in action_list:
+            for item in block:
+                if action in item:
+                    if 'protocols' in item[action]:
+                        caller = (item[action]['protocols'][LxcProtocol.name])
+                        for call in caller:
+                            self.assertIn(call['request'], pair)
 
     @unittest.skipIf(infrastructure_error('lxc-info'), "lxc-info not installed")
     def test_fastboot_lxc(self):
@@ -163,11 +181,11 @@ class TestFastbootDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
         job.validate()
         self.assertEqual(
             {
-                '1.7.3.20': '4_android-optee',
-                '1.7.3.4': '0_get-adb-serial',
-                '1.7.3.12': '2_android-busybox',
-                '1.7.3.8': '1_android-meminfo',
-                '1.7.3.16': '3_android-ping-dns'},
+                '1.8.3.20': '4_android-optee',
+                '1.8.3.4': '0_get-adb-serial',
+                '1.8.3.12': '2_android-busybox',
+                '1.8.3.8': '1_android-meminfo',
+                '1.8.3.16': '3_android-ping-dns'},
             testdef.get_namespace_data(action='test-runscript-overlay', label='test-runscript-overlay', key='testdef_levels'))
         for testdef in testdef.test_list[0]:
             self.assertEqual('git', testdef['from'])
@@ -188,7 +206,7 @@ class TestFastbootDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
             self.assertIsNotNone(action.name)
             if isinstance(action, DeployAction):
                 if action.parameters['namespace'] == 'tlxc':
-                    overlay = action.pipeline.actions[6]
+                    overlay = action.pipeline.actions[7]
         self.assertIsNotNone(overlay)
         # these tests require that lava-dispatcher itself is installed, not just running tests from a git clone
         self.assertTrue(os.path.exists(overlay.lava_test_dir))
@@ -227,8 +245,6 @@ class TestFastbootDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
         description_ref = self.pipeline_reference('db410c.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
         boot = [action for action in job.pipeline.actions if action.name == 'fastboot-boot'][0]
-        wait = [action for action in boot.internal_pipeline.actions if action.name == 'wait-usb-device'][0]
-        self.assertEqual(wait.device_actions, ['remove'])
 
     def test_x15_job(self):
         self.factory = FastBootFactory()
